@@ -176,15 +176,19 @@ soup_body_input_stream_read_chunked (SoupBodyInputStream  *bistream,
 again:
 	switch (priv->chunked_state) {
 	case SOUP_BODY_INPUT_STREAM_STATE_CHUNK_SIZE:
-		nread = soup_filter_input_stream_read_line (
-			fstream, metabuf, sizeof (metabuf), blocking,
+		nread = soup_filter_input_stream_read_until (
+			fstream, metabuf, sizeof (metabuf),
+                        "\r\n", 2, blocking, TRUE,
 			&got_line, cancellable, error);
-		if (nread <= 0)
+		if (nread < 0)
 			return nread;
-		if (!got_line) {
-			g_set_error_literal (error, G_IO_ERROR,
-					     G_IO_ERROR_PARTIAL_INPUT,
-					     _("Connection terminated unexpectedly"));
+
+		if (nread == 0 || !got_line) {
+			if (error && *error == NULL) {
+				g_set_error_literal (error, G_IO_ERROR,
+						     G_IO_ERROR_PARTIAL_INPUT,
+						     _("Connection terminated unexpectedly"));
+			}
 			return -1;
 		}
 
@@ -208,16 +212,19 @@ again:
 		return nread;
 
 	case SOUP_BODY_INPUT_STREAM_STATE_CHUNK_END:
-		nread = soup_filter_input_stream_read_line (
+		nread = soup_filter_input_stream_read_until (
 			SOUP_FILTER_INPUT_STREAM (priv->base_stream),
-			metabuf, sizeof (metabuf), blocking,
+			metabuf, sizeof (metabuf),
+                        "\r\n", 2, blocking, TRUE,
 			&got_line, cancellable, error);
-		if (nread <= 0)
+		if (nread < 0)
 			return nread;
-		if (!got_line) {
-			g_set_error_literal (error, G_IO_ERROR,
-					     G_IO_ERROR_PARTIAL_INPUT,
-					     _("Connection terminated unexpectedly"));
+		if (nread == 0 || !got_line) {
+			if (error && *error == NULL) {
+				g_set_error_literal (error, G_IO_ERROR,
+						     G_IO_ERROR_PARTIAL_INPUT,
+						     _("Connection terminated unexpectedly"));
+			}
 			return -1;
 		}
 
@@ -225,13 +232,23 @@ again:
 		break;
 
 	case SOUP_BODY_INPUT_STREAM_STATE_TRAILERS:
-		nread = soup_filter_input_stream_read_line (
-			fstream, buffer, count, blocking,
+		nread = soup_filter_input_stream_read_until (
+			fstream, metabuf, sizeof (metabuf),
+                        "\r\n", 2, blocking, TRUE,
 			&got_line, cancellable, error);
-		if (nread <= 0)
+		if (nread < 0)
 			return nread;
 
-		if (strncmp (buffer, "\r\n", nread) || strncmp (buffer, "\n", nread)) {
+		if (nread == 0) {
+			if (error && *error == NULL) {
+				g_set_error_literal (error, G_IO_ERROR,
+						     G_IO_ERROR_PARTIAL_INPUT,
+						     _("Connection terminated unexpectedly"));
+			}
+			return -1;
+		}
+
+		if (nread == 2 && strncmp (metabuf, "\r\n", nread) == 0) {
 			priv->chunked_state = SOUP_BODY_INPUT_STREAM_STATE_DONE;
 			priv->eof = TRUE;
 		}
@@ -277,8 +294,13 @@ read_internal (GInputStream  *stream,
 
 		nread = soup_body_input_stream_read_raw (bistream, buffer, count,
 							 blocking, cancellable, error);
-		if (priv->read_length != -1 && nread > 0)
-			priv->read_length -= nread;
+		if (priv->read_length != -1 && nread > 0) {
+		        priv->read_length -= nread;
+
+		        if (priv->encoding == SOUP_ENCODING_CONTENT_LENGTH && priv->read_length == 0) {
+		                priv->eof = TRUE;
+		        }
+		}
 
 		if (priv->encoding == SOUP_ENCODING_CONTENT_LENGTH)
 			priv->pos += nread;
