@@ -139,7 +139,8 @@ soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
 		for (p = strchr (value, '\r'); p; p = strchr (p, '\r'))
 			*p = ' ';
 
-		soup_message_headers_append_untrusted_data (dest, name, value);
+		if (!soup_message_headers_append_untrusted_data (dest, name, value))
+                        goto done;
         }
 	success = TRUE;
 
@@ -240,7 +241,12 @@ soup_headers_parse_request (const char          *str,
 	if (headers >= str + len || *headers != '\n')
 		return SOUP_STATUS_BAD_REQUEST;
 
-	if (!soup_headers_parse (str, len, req_headers)) 
+	// Ensure pointer location for Request-Line end matches location of 'headers'
+	p = strchr(str, '\n');
+	if (p != headers)
+		return SOUP_STATUS_BAD_REQUEST;
+
+	if (!soup_headers_parse (str, len, req_headers))
 		return SOUP_STATUS_BAD_REQUEST;
 
 	if (soup_message_headers_get_expectations (req_headers) &
