@@ -23,24 +23,25 @@
 #include "soup-session-feature-private.h"
 
 /**
- * SECTION:soup-logger
- * @short_description: Debug logging support
+ * SoupLogger:
  *
- * #SoupLogger watches a #SoupSession and logs the HTTP traffic that
+ * Debug logging support
+ *
+ * [class@Logger] watches a [class@Session] and logs the HTTP traffic that
  * it generates, for debugging purposes. Many applications use an
  * environment variable to determine whether or not to use
- * #SoupLogger, and to determine the amount of debugging output.
+ * [class@Logger], and to determine the amount of debugging output.
  *
- * To use #SoupLogger, first create a logger with soup_logger_new(),
- * optionally configure it with soup_logger_set_request_filter(),
- * soup_logger_set_response_filter(), and soup_logger_set_printer(),
- * and then attach it to a session (or multiple sessions) with
- * soup_session_add_feature().
+ * To use [class@Logger], first create a logger with [ctor@Logger.new], optionally
+ * configure it with [method@Logger.set_request_filter],
+ * [method@Logger.set_response_filter], and [method@Logger.set_printer], and
+ * then attach it to a session (or multiple sessions) with
+ * [method@Session.add_feature].
  *
- * By default, the debugging output is sent to
- * <literal>stdout</literal>, and looks something like:
+ * By default, the debugging output is sent to `stdout`, and looks something
+ * like:
  *
- * <informalexample><screen>
+ * ```
  * > POST /unauth HTTP/1.1
  * > Soup-Debug-Timestamp: 1200171744
  * > Soup-Debug: SoupSession 1 (0x612190), SoupMessage 1 (0x617000), GSocket 1 (0x612220)
@@ -48,60 +49,51 @@
  * > Content-Type: text/plain
  * > Connection: close
  *
- * &lt; HTTP/1.1 201 Created
- * &lt; Soup-Debug-Timestamp: 1200171744
- * &lt; Soup-Debug: SoupMessage 1 (0x617000)
- * &lt; Date: Sun, 12 Jan 2008 21:02:24 GMT
- * &lt; Content-Length: 0
- * </screen></informalexample>
+ * < HTTP/1.1 201 Created
+ * < Soup-Debug-Timestamp: 1200171744
+ * < Soup-Debug: SoupMessage 1 (0x617000)
+ * < Date: Sun, 12 Jan 2008 21:02:24 GMT
+ * < Content-Length: 0
+ * ```
  *
- * The <literal>Soup-Debug-Timestamp</literal> line gives the time (as
- * a <type>time_t</type>) when the request was sent, or the response fully
- * received.
+ * The `Soup-Debug-Timestamp` line gives the time (as a `time_t`) when the
+ * request was sent, or the response fully received.
  *
- * The <literal>Soup-Debug</literal> line gives further debugging
- * information about the #SoupSession, #SoupMessage, and #GSocket
- * involved; the hex numbers are the addresses of the objects in
- * question (which may be useful if you are running in a debugger).
- * The decimal IDs are simply counters that uniquely identify objects
- * across the lifetime of the #SoupLogger. In particular, this can be
- * used to identify when multiple messages are sent across the same
- * connection.
+ * The `Soup-Debug` line gives further debugging information about the
+ * [class@Session], [class@Message], and [class@Gio.Socket] involved; the hex
+ * numbers are the addresses of the objects in question (which may be useful if
+ * you are running in a debugger). The decimal IDs are simply counters that
+ * uniquely identify objects across the lifetime of the [class@Logger]. In
+ * particular, this can be used to identify when multiple messages are sent
+ * across the same connection.
  *
  * Currently, the request half of the message is logged just before
  * the first byte of the request gets written to the network (from the
- * #SoupMessage::starting signal).
+ * [signal@Message::starting] signal).
  *
- * The response is logged just after the last byte of the response
- * body is read from the network (from the #SoupMessage::got-body or
- * #SoupMessage::got-informational signal), which means that the
- * #SoupMessage::got-headers signal, and anything triggered off it
- * (such as #SoupMessage::authenticate) will be emitted
- * <emphasis>before</emphasis> the response headers are actually
- * logged.
+ * The response is logged just after the last byte of the response body is read
+ * from the network (from the [signal@Message::got-body] or
+ * [signal@Message::got-informational] signal), which means that the
+ * [signal@Message::got-headers] signal, and anything triggered off it (such as
+ * [signal@Message::authenticate]) will be emitted *before* the response headers are
+ * actually logged.
  *
- * If the response doesn't happen to trigger the
- * #SoupMessage::got-body nor #SoupMessage::got-informational signals
- * due to, for example, a cancellation before receiving the last byte
- * of the response body, the response will still be logged on the
- * event of the #SoupMessage::finished signal.
+ * If the response doesn't happen to trigger the [signal@Message::got-body] nor
+ * [signal@Message::got-informational] signals due to, for example, a
+ * cancellation before receiving the last byte of the response body, the
+ * response will still be logged on the event of the [signal@Message::finished]
+ * signal.
  **/
 
-/**
- * SoupLogger:
- *
- * Class implementing logging.
- */
-
 struct _SoupLogger {
 	GObject parent;
 };
 
 typedef struct {
 	GQuark              tag;
+        GMutex              mutex;
 	GHashTable         *ids;
 	GHashTable         *request_bodies;
-	GHashTable         *request_messages;
 	GHashTable         *response_bodies;
 
 	SoupSession        *session;
@@ -132,8 +124,6 @@ enum {
 
 static GParamSpec *properties[LAST_PROPERTY] = { NULL, };
 
-static void body_ostream_done (gpointer data, GObject *bostream);
-
 static void soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 
 static SoupContentProcessorInterface *soup_logger_default_content_processor_interface;
@@ -156,12 +146,14 @@ write_body (SoupLogger *logger, const char *buffer, gsize nread,
         if (!nread)
                 return;
 
+        g_mutex_lock (&priv->mutex);
         body = g_hash_table_lookup (bodies, key);
 
         if (!body) {
             body = g_string_new (NULL);
             g_hash_table_insert (bodies, key, body);
         }
+        g_mutex_unlock (&priv->mutex);
 
         if (priv->max_body_size >= 0) {
                 /* longer than max => we've written the extra [...] */
@@ -204,11 +196,14 @@ soup_logger_content_processor_wrap_input (SoupContentProcessor *processor,
         SoupLogger *logger = SOUP_LOGGER (processor);
         SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
         SoupLoggerInputStream *stream;
-        SoupLoggerLogLevel log_level;
+        SoupLoggerLogLevel log_level = SOUP_LOGGER_LOG_NONE;
 
-        if (priv->request_filter)
-                log_level = priv->request_filter (logger, msg,
-                                                  priv->response_filter_data);
+        if (priv->request_filter || priv->response_filter) {
+                if (priv->request_filter)
+                        log_level = priv->request_filter (logger, msg, priv->request_filter_data);
+                if (priv->response_filter)
+                        log_level = MAX(log_level, priv->response_filter (logger, msg, priv->response_filter_data));
+        }
         else
                 log_level = priv->level;
 
@@ -255,13 +250,7 @@ soup_logger_init (SoupLogger *logger)
 	priv->ids = g_hash_table_new (NULL, NULL);
 	priv->request_bodies = g_hash_table_new_full (NULL, NULL, NULL, body_free);
 	priv->response_bodies = g_hash_table_new_full (NULL, NULL, NULL, body_free);
-	priv->request_messages = g_hash_table_new (NULL, NULL);
-}
-
-static void
-body_ostream_drop_ref (gpointer key, gpointer value, gpointer data)
-{
-        g_object_weak_unref (key, body_ostream_done, data);
+        g_mutex_init (&priv->mutex);
 }
 
 static void
@@ -270,13 +259,9 @@ soup_logger_finalize (GObject *object)
 	SoupLogger *logger = SOUP_LOGGER (object);
 	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
 
-	g_hash_table_foreach (priv->request_messages,
-	                      body_ostream_drop_ref, priv);
-
 	g_hash_table_destroy (priv->ids);
 	g_hash_table_destroy (priv->request_bodies);
 	g_hash_table_destroy (priv->response_bodies);
-	g_hash_table_destroy (priv->request_messages);
 
 	if (priv->request_filter_dnotify)
 		priv->request_filter_dnotify (priv->request_filter_data);
@@ -285,6 +270,8 @@ soup_logger_finalize (GObject *object)
 	if (priv->printer_dnotify)
 		priv->printer_dnotify (priv->printer_data);
 
+        g_mutex_clear (&priv->mutex);
+
 	G_OBJECT_CLASS (soup_logger_parent_class)->finalize (object);
 }
 
@@ -341,8 +328,7 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
 	/**
 	 * SoupLogger:level:
 	 *
-	 * The level of logging output
-	 *
+	 * The level of logging output.
 	 */
         properties[PROP_LEVEL] =
 		g_param_spec_enum ("level",
@@ -353,12 +339,11 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
 				    G_PARAM_READWRITE |
 				    G_PARAM_STATIC_STRINGS);
 	/**
-	 * SoupLogger:max-body-size:
+	 * SoupLogger:max-body-size: (attributes org.gtk.Property.get=soup_logger_get_max_body_size org.gtk.Property.set=soup_logger_set_max_body_size)
 	 *
-	 * If #SoupLogger:level is %SOUP_LOGGER_LOG_BODY, this gives
+	 * If [property@Logger:level] is %SOUP_LOGGER_LOG_BODY, this gives
 	 * the maximum number of bytes of the body that will be logged.
 	 * (-1 means "no limit".)
-	 *
 	 **/
         properties[PROP_MAX_BODY_SIZE] =
 		g_param_spec_int ("max-body-size",
@@ -378,7 +363,7 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * SoupLoggerLogLevel:
  * @SOUP_LOGGER_LOG_NONE: No logging
  * @SOUP_LOGGER_LOG_MINIMAL: Log the Request-Line or Status-Line and
- * the Soup-Debug pseudo-headers
+ *   the Soup-Debug pseudo-headers
  * @SOUP_LOGGER_LOG_HEADERS: Log the full request/response headers
  * @SOUP_LOGGER_LOG_BODY: Log the full headers and request/response bodies
  *
@@ -389,11 +374,11 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * soup_logger_new:
  * @level: the debug level
  *
- * Creates a new #SoupLogger with the given debug level.
+ * Creates a new [class@Logger] with the given debug level.
  *
  * If you need finer control over what message parts are and aren't
- * logged, use soup_logger_set_request_filter() and
- * soup_logger_set_response_filter().
+ * logged, use [method@Logger.set_request_filter] and
+ * [method@Logger.set_response_filter].
  *
  * Returns: a new #SoupLogger
  **/
@@ -407,16 +392,17 @@ soup_logger_new (SoupLoggerLogLevel level)
  * SoupLoggerFilter:
  * @logger: the #SoupLogger
  * @msg: the message being logged
- * @user_data: the data passed to soup_logger_set_request_filter()
- * or soup_logger_set_response_filter()
+ * @user_data: the data passed to [method@Logger.set_request_filter]
+ *   or [method@Logger.set_response_filter]
  *
- * The prototype for a logging filter. The filter callback will be
- * invoked for each request or response, and should analyze it and
- * return a #SoupLoggerLogLevel value indicating how much of the
- * message to log.
+ * The prototype for a logging filter.
  *
- * Returns: a #SoupLoggerLogLevel value indicating how much of
- * the message to log
+ * The filter callback will be invoked for each request or response, and should
+ * analyze it and return a [enum@LoggerLogLevel] value indicating how much of
+ * the message to log.
+ *
+ * Returns: a [enum@LoggerLogLevel] value indicating how much of the message to
+ *   log
  **/
 
 /**
@@ -427,10 +413,11 @@ soup_logger_new (SoupLoggerLogLevel level)
  * @destroy: a #GDestroyNotify to free @filter_data
  *
  * Sets up a filter to determine the log level for a given request.
+ *
  * For each HTTP request @logger will invoke @request_filter to
  * determine how much (if any) of that request to log. (If you do not
  * set a request filter, @logger will just always log requests at the
- * level passed to soup_logger_new().)
+ * level passed to [ctor@Logger.new].)
  **/
 void
 soup_logger_set_request_filter (SoupLogger       *logger,
@@ -453,10 +440,11 @@ soup_logger_set_request_filter (SoupLogger       *logger,
  * @destroy: a #GDestroyNotify to free @filter_data
  *
  * Sets up a filter to determine the log level for a given response.
+ *
  * For each HTTP response @logger will invoke @response_filter to
  * determine how much (if any) of that response to log. (If you do not
  * set a response filter, @logger will just always log responses at
- * the level passed to soup_logger_new().)
+ * the level passed to [ctor@Logger.new].)
  **/
 void
 soup_logger_set_response_filter (SoupLogger       *logger,
@@ -477,7 +465,7 @@ soup_logger_set_response_filter (SoupLogger       *logger,
  * @level: the level of the information being printed.
  * @direction: a single-character prefix to @data
  * @data: data to print
- * @user_data: the data passed to soup_logger_set_printer()
+ * @user_data: the data passed to [method@Logger.set_printer]
  *
  * The prototype for a custom printing callback.
  *
@@ -489,9 +477,9 @@ soup_logger_set_response_filter (SoupLogger       *logger,
  *
  * To get the effect of the default printer, you would do:
  *
- * <informalexample><programlisting>
- *	printf ("%c %s\n", direction, data);
- * </programlisting></informalexample>
+ * ```c
+ * printf ("%c %s\n", direction, data);
+ * ```
  **/
 
 /**
@@ -502,7 +490,7 @@ soup_logger_set_response_filter (SoupLogger       *logger,
  * @destroy: a #GDestroyNotify to free @printer_data
  *
  * Sets up an alternate log printing routine, if you don't want
- * the log to go to <literal>stdout</literal>.
+ * the log to go to `stdout`.
  **/
 void
 soup_logger_set_printer (SoupLogger        *logger,
@@ -518,7 +506,7 @@ soup_logger_set_printer (SoupLogger        *logger,
 }
 
 /**
- * soup_logger_set_max_body_size:
+ * soup_logger_set_max_body_size: (attributes org.gtk.Method.set_property=max-body-size)
  * @logger: a #SoupLogger
  * @max_body_size: the maximum body size to log
  *
@@ -533,7 +521,7 @@ soup_logger_set_max_body_size (SoupLogger *logger, int max_body_size)
 }
 
 /**
- * soup_logger_get_max_body_size:
+ * soup_logger_get_max_body_size: (attributes org.gtk.Method.get_property=max-body-size)
  * @logger: a #SoupLogger
  *
  * Get the maximum body size for @logger.
@@ -563,9 +551,11 @@ soup_logger_set_id (SoupLogger *logger, gpointer object)
 	gpointer klass = G_OBJECT_GET_CLASS (object);
 	gpointer id;
 
+        g_mutex_lock (&priv->mutex);
 	id = g_hash_table_lookup (priv->ids, klass);
 	id = (char *)id + 1;
 	g_hash_table_insert (priv->ids, klass, id);
+        g_mutex_unlock (&priv->mutex);
 
 	g_object_set_qdata (object, priv->tag, id);
 	return GPOINTER_TO_UINT (id);
@@ -603,6 +593,15 @@ soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
 	g_free (data);
 }
 
+static void
+soup_logger_print_decorative_newline (SoupLogger *logger)
+{
+	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	if (!priv->printer) {
+		g_print("\n");
+	}
+}
+
 static void
 soup_logger_print_basic_auth (SoupLogger *logger, const char *value)
 {
@@ -693,6 +692,8 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
+	soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>', "Soup-Host: %s", g_uri_get_host (uri));
+
 	soup_message_headers_iter_init (&iter, soup_message_get_request_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
 		if (!g_ascii_strcasecmp (name, "Authorization") &&
@@ -705,17 +706,20 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	}
 
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		return;
+		goto print_newline_and_return;
 
 	/* will be logged in get_informational */
 	if (soup_message_headers_get_expectations (soup_message_get_request_headers (msg)) == SOUP_EXPECTATION_CONTINUE)
-		return;
+		goto print_newline_and_return;
 
 	if (!g_hash_table_steal_extended (priv->request_bodies, msg, NULL, (gpointer *)&body))
-		return;
+		goto print_newline_and_return;
 
 	soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '>', "\n%s", body->str);
 	g_string_free (body, TRUE);
+
+print_newline_and_return:
+	soup_logger_print_decorative_newline (logger);
 }
 
 static void
@@ -750,7 +754,7 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 			   soup_logger_get_id (logger, msg), msg);
 
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
-		return;
+		goto print_newline_and_return;
 
 	soup_message_headers_iter_init (&iter, soup_message_get_response_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
@@ -759,19 +763,23 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	}
 
 	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		return;
+		goto print_newline_and_return;
 
 	if (!g_hash_table_steal_extended (priv->response_bodies, msg, NULL, (gpointer *)&body))
-		return;
+		goto print_newline_and_return;
 
 	soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '<', "\n%s", body->str);
 	g_string_free (body, TRUE);
+
+print_newline_and_return:
+	soup_logger_print_decorative_newline (logger);
 }
 
 static void
 finished (SoupMessage *msg, gpointer user_data)
 {
 	SoupLogger *logger = user_data;
+        SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
 
         /* Do not print the response if we didn't print a request. This can happen if
          * msg is a preconnect request, for example.
@@ -779,8 +787,9 @@ finished (SoupMessage *msg, gpointer user_data)
         if (!soup_logger_get_id (logger, msg))
                 return;
 
+        g_mutex_lock (&priv->mutex);
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
@@ -791,6 +800,8 @@ got_informational (SoupMessage *msg, gpointer user_data)
         SoupLoggerLogLevel log_level;
         GString *body = NULL;
 
+        g_mutex_lock (&priv->mutex);
+
         if (priv->response_filter)
                 log_level = priv->response_filter (logger, msg,
                                                    priv->response_filter_data);
@@ -799,10 +810,11 @@ got_informational (SoupMessage *msg, gpointer user_data)
 
         g_signal_handlers_disconnect_by_func (msg, finished, logger);
         print_response (logger, msg);
-        soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
-        if (!g_hash_table_steal_extended (priv->response_bodies, msg, NULL, (gpointer *)&body))
+        if (!g_hash_table_steal_extended (priv->response_bodies, msg, NULL, (gpointer *)&body)) {
+                g_mutex_unlock (&priv->mutex);
                 return;
+        }
 
         if (soup_message_get_status (msg) == SOUP_STATUS_CONTINUE) {
                 soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
@@ -813,61 +825,27 @@ got_informational (SoupMessage *msg, gpointer user_data)
                                            '>', "%s", body->str);
                 }
 
-                soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+                soup_logger_print_decorative_newline (logger);
         }
 
         g_string_free (body, TRUE);
+
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
 got_body (SoupMessage *msg, gpointer user_data)
 {
 	SoupLogger *logger = user_data;
+        SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+
+        g_mutex_lock (&priv->mutex);
 
 	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
-}
 
-static void
-body_stream_wrote_data_cb (GOutputStream *stream,
-                           const void    *buffer,
-                           guint          count,
-                           gboolean       is_metadata,
-                           SoupLogger    *logger)
-{
-        SoupLoggerPrivate *priv;
-        SoupMessage *msg;
-
-        if (is_metadata)
-                return;
-
-        priv = soup_logger_get_instance_private (logger);
-        msg = g_hash_table_lookup (priv->request_messages, stream);
-        write_body (logger, buffer, count, msg, priv->request_bodies);
-}
-
-static void
-body_ostream_done (gpointer data, GObject *bostream)
-{
-        SoupLoggerPrivate *priv = data;
-
-        g_hash_table_remove (priv->request_messages, bostream);
-}
-
-void
-soup_logger_request_body_setup (SoupLogger           *logger,
-                                SoupMessage          *msg,
-                                SoupBodyOutputStream *stream)
-{
-        SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
-
-        g_hash_table_insert (priv->request_messages, stream, msg);
-        g_signal_connect_object (stream, "wrote-data",
-                                 G_CALLBACK (body_stream_wrote_data_cb),
-                                 logger, 0);
-        g_object_weak_ref (G_OBJECT (stream), body_ostream_done, priv);
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
@@ -878,7 +856,7 @@ wrote_body (SoupMessage *msg, gpointer user_data)
 	gboolean restarted;
 	guint msg_id;
 	SoupConnection *conn;
-	GSocket *socket;
+	GSocket *socket = NULL;
 
 	msg_id = soup_logger_get_id (logger, msg);
 	if (msg_id)
@@ -892,12 +870,16 @@ wrote_body (SoupMessage *msg, gpointer user_data)
 		soup_logger_set_id (logger, priv->session);
 
 	conn = soup_message_get_connection (msg);
-	socket = conn ? soup_connection_get_socket (conn) : NULL;
+        if (conn) {
+                socket = soup_connection_get_socket (conn);
+                g_object_unref (conn);
+        }
 	if (socket && !soup_logger_get_id (logger, socket))
 		soup_logger_set_id (logger, socket);
 
+        g_mutex_lock (&priv->mutex);
 	print_request (logger, msg, socket, restarted);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
+        g_mutex_unlock (&priv->mutex);
 }
 
 static void
