@@ -62,7 +62,7 @@ struct _SoupMessageHeaders {
  * soup_message_headers_new:
  * @type: the type of headers
  *
- * Creates a #SoupMessageHeaders.
+ * Creates a [struct@MessageHeaders].
  *
  * ([class@Message] does this automatically for its own headers. You would only
  * need to use this method if you are manually parsing or generating message
@@ -150,19 +150,8 @@ soup_message_headers_set (SoupMessageHeaders *hdrs,
 {
         switch (name) {
         case SOUP_HEADER_CONTENT_LENGTH:
-                if (hdrs->encoding == SOUP_ENCODING_CHUNKED)
-                        return;
-
-                if (value) {
-                        char *end;
-
-                        hdrs->content_length = g_ascii_strtoull (value, &end, 10);
-                        if (*end)
-                                hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
-                        else
-                                hdrs->encoding = SOUP_ENCODING_CONTENT_LENGTH;
-                } else
-                        hdrs->encoding = -1;
+        case SOUP_HEADER_TRANSFER_ENCODING:
+                hdrs->encoding = -1;
                 break;
         case SOUP_HEADER_CONTENT_TYPE:
                 g_clear_pointer (&hdrs->content_type, g_free);
@@ -188,21 +177,6 @@ soup_message_headers_set (SoupMessageHeaders *hdrs,
                 } else
                         hdrs->expectations = 0;
                 break;
-        case SOUP_HEADER_TRANSFER_ENCODING:
-                if (value) {
-                        /* "identity" is a wrong value according to RFC errata 408,
-                         * and RFC 7230 does not list it as valid transfer-coding.
-                         * Nevertheless, the obsolete RFC 2616 stated "identity"
-                         * as valid, so we can't handle it as unrecognized here
-                         * for compatibility reasons.
-                         */
-                        if (g_ascii_strcasecmp (value, "chunked") == 0)
-                                hdrs->encoding = SOUP_ENCODING_CHUNKED;
-                        else if (g_ascii_strcasecmp (value, "identity") != 0)
-                                hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
-                } else
-                        hdrs->encoding = -1;
-                break;
         default:
                 break;
         }
@@ -273,12 +247,32 @@ soup_message_headers_clean_connection_headers (SoupMessageHeaders *hdrs)
 	soup_header_free_list (tokens);
 }
 
-void
-soup_message_headers_append_common (SoupMessageHeaders *hdrs,
-                                    SoupHeaderName      name,
-                                    const char         *value)
+static inline gboolean is_valid_header_name (const char *name)
+{
+        return name && *name && strpbrk (name, " \t\r\n:") == NULL;
+}
+
+static inline gboolean is_valid_header_value (const char *value)
+{
+        return value && strpbrk (value, "\r\n") == NULL;
+}
+
+gboolean
+soup_message_headers_append_common (SoupMessageHeaders    *hdrs,
+                                    SoupHeaderName         name,
+                                    const char            *value,
+                                    SoupHeaderValueTrusted trusted_value)
 {
         SoupCommonHeader header;
+        if (name == SOUP_HEADER_HOST && soup_message_headers_get_one (hdrs, "Host")) {
+                g_warning ("soup_message_headers_append_common: Rejecting duplicate Host header");
+                return FALSE;
+        }
+
+        if (trusted_value == SOUP_HEADER_VALUE_UNTRUSTED && !is_valid_header_value (value)) {
+                g_warning ("soup_message_headers_append: Rejecting bad value '%s'", value);
+                return FALSE;
+        }
 
         if (!hdrs->common_headers)
                 hdrs->common_headers = g_array_sized_new (FALSE, FALSE, sizeof (SoupCommonHeader), 6);
@@ -290,6 +284,43 @@ soup_message_headers_append_common (SoupMessageHeaders *hdrs,
                 g_hash_table_remove (hdrs->common_concat, GUINT_TO_POINTER (header.name));
 
         soup_message_headers_set (hdrs, name, value);
+        return TRUE;
+}
+
+static gboolean
+soup_message_headers_append_internal (SoupMessageHeaders *hdrs,
+			              const char *name, const char *value)
+{
+	SoupUncommonHeader header;
+        SoupHeaderName header_name;
+
+	g_return_val_if_fail (hdrs, FALSE);
+	g_return_val_if_fail (name != NULL, FALSE);
+	g_return_val_if_fail (value != NULL, FALSE);
+
+	if (!is_valid_header_name (name)) {
+		g_warning ("soup_message_headers_append: Rejecting bad name '%s'", name);
+		return FALSE;
+	}
+
+        header_name = soup_header_name_from_string (name);
+        if (header_name != SOUP_HEADER_UNKNOWN)
+                return soup_message_headers_append_common (hdrs, header_name, value, SOUP_HEADER_VALUE_UNTRUSTED);
+
+        if (!is_valid_header_value (value)) {
+                g_warning ("soup_message_headers_append: Rejecting bad value '%s'", value);
+                return FALSE;
+        }
+
+        if (!hdrs->uncommon_headers)
+                hdrs->uncommon_headers = g_array_sized_new (FALSE, FALSE, sizeof (SoupUncommonHeader), 6);
+
+	header.name = g_strdup (name);
+	header.value = g_strdup (value);
+	g_array_append_val (hdrs->uncommon_headers, header);
+	if (hdrs->uncommon_concat)
+		g_hash_table_remove (hdrs->uncommon_concat, header.name);
+        return TRUE;
 }
 
 /**
@@ -311,70 +342,36 @@ void
 soup_message_headers_append (SoupMessageHeaders *hdrs,
 			     const char *name, const char *value)
 {
-	SoupUncommonHeader header;
-        SoupHeaderName header_name;
-
-	g_return_if_fail (hdrs);
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (value != NULL);
-
-	/* Setting a syntactically invalid header name or value is
-	 * considered to be a programming error. However, it can also
-	 * be a security hole, so we want to fail here even if
-	 * compiled with G_DISABLE_CHECKS.
-	 */
-#ifndef G_DISABLE_CHECKS
-	g_return_if_fail (*name && strpbrk (name, " \t\r\n:") == NULL);
-	g_return_if_fail (strpbrk (value, "\r\n") == NULL);
-#else
-	if (*name && strpbrk (name, " \t\r\n:")) {
-		g_warning ("soup_message_headers_append: Ignoring bad name '%s'", name);
-		return;
-	}
-	if (strpbrk (value, "\r\n")) {
-		g_warning ("soup_message_headers_append: Ignoring bad value '%s'", value);
-		return;
-	}
-#endif
-
-        header_name = soup_header_name_from_string (name);
-        if (header_name != SOUP_HEADER_UNKNOWN) {
-                soup_message_headers_append_common (hdrs, header_name, value);
-                return;
-        }
-
-        if (!hdrs->uncommon_headers)
-                hdrs->uncommon_headers = g_array_sized_new (FALSE, FALSE, sizeof (SoupUncommonHeader), 6);
-
-	header.name = g_strdup (name);
-	header.value = g_strdup (value);
-	g_array_append_val (hdrs->uncommon_headers, header);
-	if (hdrs->uncommon_concat)
-		g_hash_table_remove (hdrs->uncommon_concat, header.name);
+	soup_message_headers_append_internal (hdrs, name, value);
 }
 
 /*
- * Appends a header value ensuring that it is valid UTF8.
+ * Appends a header value ensuring that it is valid UTF-8, and also checking the
+ * return value of soup_message_headers_append_internal() to report whether the
+ * headers are invalid for various other reasons.
  */
-void
+gboolean
 soup_message_headers_append_untrusted_data (SoupMessageHeaders *hdrs,
                                             const char         *name,
                                             const char         *value)
 {
         char *safe_value = g_utf8_make_valid (value, -1);
         char *safe_name = g_utf8_make_valid (name, -1);
-        soup_message_headers_append (hdrs, safe_name, safe_value);
+        gboolean result = soup_message_headers_append_internal (hdrs, safe_name, safe_value);
+
         g_free (safe_value);
         g_free (safe_name);
+        return result;
 }
 
 void
-soup_message_headers_replace_common (SoupMessageHeaders *hdrs,
-                                     SoupHeaderName      name,
-                                     const char         *value)
+soup_message_headers_replace_common (SoupMessageHeaders    *hdrs,
+                                     SoupHeaderName         name,
+                                     const char            *value,
+                                     SoupHeaderValueTrusted trusted_value)
 {
         soup_message_headers_remove_common (hdrs, name);
-        soup_message_headers_append_common (hdrs, name, value);
+        soup_message_headers_append_common (hdrs, name, value, trusted_value);
 }
 
 /**
@@ -780,11 +777,10 @@ soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
 /**
  * SoupMessageHeadersIter:
  *
- * An opaque type used to iterate over a %SoupMessageHeaders
- * structure.
+ * An opaque type used to iterate over a [struct@MessageHeaders] structure
  *
  * After intializing the iterator with [func@MessageHeadersIter.init], call
- * [method@MessageHeadersIter.next] to fetch data from it.
+ * [func@MessageHeadersIter.next] to fetch data from it.
  *
  * You may not modify the headers while iterating over them.
  **/
@@ -797,9 +793,8 @@ typedef struct {
 
 /**
  * soup_message_headers_iter_init:
- * @iter: (out) (transfer none): a pointer to a %SoupMessageHeadersIter
- *   structure
- * @hdrs: a %SoupMessageHeaders
+ * @iter: (out) (transfer none): a pointer to a #SoupMessageHeadersIter structure
+ * @hdrs: a #SoupMessageHeaders
  *
  * Initializes @iter for iterating @hdrs.
  **/
@@ -816,7 +811,7 @@ soup_message_headers_iter_init (SoupMessageHeadersIter *iter,
 
 /**
  * soup_message_headers_iter_next:
- * @iter: (inout) (transfer none): a %SoupMessageHeadersIter
+ * @iter: (inout) (transfer none): a #SoupMessageHeadersIter
  * @name: (out) (transfer none): pointer to a variable to return
  *   the header name in
  * @value: (out) (transfer none): pointer to a variable to return
@@ -826,7 +821,7 @@ soup_message_headers_iter_init (SoupMessageHeadersIter *iter,
  * iterated by @iter.
  *
  * If @iter has already yielded the last header, then
- * [method@MessageHeadersIter.next] will return %FALSE and @name and @value
+ * [func@MessageHeadersIter.next] will return %FALSE and @name and @value
  * will be unchanged.
  *
  * Returns: %TRUE if another name and value were returned, %FALSE
@@ -947,32 +942,52 @@ soup_message_headers_foreach (SoupMessageHeaders           *hdrs,
 SoupEncoding
 soup_message_headers_get_encoding (SoupMessageHeaders *hdrs)
 {
-	const char *header;
+	const char *content_length;
+        const char *transfer_encoding;
 
 	g_return_val_if_fail (hdrs, SOUP_ENCODING_UNRECOGNIZED);
 
 	if (hdrs->encoding != -1)
 		return hdrs->encoding;
 
-	/* If Transfer-Encoding was set, hdrs->encoding would already
-	 * be set. So we don't need to check that possibility.
-	 */
-	header = soup_message_headers_get_one_common (hdrs, SOUP_HEADER_CONTENT_LENGTH);
-	if (header) {
-                soup_message_headers_set (hdrs, SOUP_HEADER_CONTENT_LENGTH, header);
-		if (hdrs->encoding != -1)
-			return hdrs->encoding;
-	}
+        /* Transfer-Encoding is checked first because it overrides the Content-Length */
+        transfer_encoding = soup_message_headers_get_one_common (hdrs, SOUP_HEADER_TRANSFER_ENCODING);
+        if (transfer_encoding) {
+                /* "identity" is a wrong value according to RFC errata 408,
+                 * and RFC 7230 does not list it as valid transfer-coding.
+                 * Nevertheless, the obsolete RFC 2616 stated "identity"
+                 * as valid, so we can't handle it as unrecognized here
+                 * for compatibility reasons.
+                 */
+                if (g_ascii_strcasecmp (transfer_encoding, "chunked") == 0)
+                        hdrs->encoding = SOUP_ENCODING_CHUNKED;
+                else if (g_ascii_strcasecmp (transfer_encoding, "identity") != 0)
+                        hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
+        } else {
+                content_length = soup_message_headers_get_one_common (hdrs, SOUP_HEADER_CONTENT_LENGTH);
+                if (content_length) {
+                        char *end;
+
+                        hdrs->content_length = g_ascii_strtoull (content_length, &end, 10);
+                        if (*end)
+                                hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
+                        else
+                                hdrs->encoding = SOUP_ENCODING_CONTENT_LENGTH;
+                }
+        }
+
+        if (hdrs->encoding == -1) {
+                /* Per RFC 2616 4.4, a response body that doesn't indicate its
+                 * encoding otherwise is terminated by connection close, and a
+                 * request that doesn't indicate otherwise has no body. Note
+                 * that SoupMessage calls soup_message_headers_set_encoding()
+                 * to override the response body default for our own
+                 * server-side messages.
+                 */
+                hdrs->encoding = (hdrs->type == SOUP_MESSAGE_HEADERS_RESPONSE) ?
+                        SOUP_ENCODING_EOF : SOUP_ENCODING_NONE;
+        }
 
-	/* Per RFC 2616 4.4, a response body that doesn't indicate its
-	 * encoding otherwise is terminated by connection close, and a
-	 * request that doesn't indicate otherwise has no body. Note
-	 * that SoupMessage calls soup_message_headers_set_encoding()
-	 * to override the response body default for our own
-	 * server-side messages.
-	 */
-	hdrs->encoding = (hdrs->type == SOUP_MESSAGE_HEADERS_RESPONSE) ?
-		SOUP_ENCODING_EOF : SOUP_ENCODING_NONE;
 	return hdrs->encoding;
 }
 
@@ -1008,7 +1023,7 @@ soup_message_headers_set_encoding (SoupMessageHeaders *hdrs,
 
 	case SOUP_ENCODING_CHUNKED:
 		soup_message_headers_remove_common (hdrs, SOUP_HEADER_CONTENT_LENGTH);
-		soup_message_headers_replace_common (hdrs, SOUP_HEADER_TRANSFER_ENCODING, "chunked");
+		soup_message_headers_replace_common (hdrs, SOUP_HEADER_TRANSFER_ENCODING, "chunked", SOUP_HEADER_VALUE_TRUSTED);
 		break;
 
 	default:
@@ -1071,7 +1086,7 @@ soup_message_headers_set_content_length (SoupMessageHeaders *hdrs,
 	g_snprintf (length, sizeof (length), "%" G_GUINT64_FORMAT,
 		    content_length);
 	soup_message_headers_remove_common (hdrs, SOUP_HEADER_TRANSFER_ENCODING);
-	soup_message_headers_replace_common (hdrs, SOUP_HEADER_CONTENT_LENGTH, length);
+	soup_message_headers_replace_common (hdrs, SOUP_HEADER_CONTENT_LENGTH, length, SOUP_HEADER_VALUE_TRUSTED);
 }
 
 /**
@@ -1124,7 +1139,7 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
 	g_return_if_fail ((expectations & ~SOUP_EXPECTATION_CONTINUE) == 0);
 
 	if (expectations & SOUP_EXPECTATION_CONTINUE)
-		soup_message_headers_replace_common (hdrs, SOUP_HEADER_EXPECT, "100-continue");
+		soup_message_headers_replace_common (hdrs, SOUP_HEADER_EXPECT, "100-continue", SOUP_HEADER_VALUE_TRUSTED);
 	else
 		soup_message_headers_remove_common (hdrs, SOUP_HEADER_EXPECT);
 }
@@ -1160,10 +1175,16 @@ sort_ranges (gconstpointer a, gconstpointer b)
 }
 
 /* like soup_message_headers_get_ranges(), except it returns:
- *   SOUP_STATUS_OK if there is no Range or it should be ignored.
- *   SOUP_STATUS_PARTIAL_CONTENT if there is at least one satisfiable range.
- *   SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE if @check_satisfiable
- *     is %TRUE and the request is not satisfiable given @total_length.
+ *  - SOUP_STATUS_OK if there is no Range or it should be ignored due to being
+ *    entirely invalid.
+ *  - SOUP_STATUS_PARTIAL_CONTENT if there is at least one satisfiable range.
+ *  - SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE if @check_satisfiable
+ *     is %TRUE, the Range is valid, but no part of the request is satisfiable
+ *     given @total_length.
+ *
+ * @ranges and @length are only set if SOUP_STATUS_PARTIAL_CONTENT is returned.
+ *
+ * See https://httpwg.org/specs/rfc9110.html#field.range
  */
 guint
 soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
@@ -1177,22 +1198,28 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
 	GArray *array;
 	char *spec, *end;
 	guint status = SOUP_STATUS_OK;
+	gboolean is_all_valid = TRUE;
 
 	if (!range || strncmp (range, "bytes", 5) != 0)
-		return status;
+		return SOUP_STATUS_OK;  /* invalid header or unknown range unit */
 
 	range += 5;
 	while (g_ascii_isspace (*range))
 		range++;
 	if (*range++ != '=')
-		return status;
+		return SOUP_STATUS_OK;  /* invalid header */
 	while (g_ascii_isspace (*range))
 		range++;
 
 	range_list = soup_header_parse_list (range);
 	if (!range_list)
-		return status;
+		return SOUP_STATUS_OK;  /* invalid list */
 
+	/* Loop through the ranges and modify the status accordingly. Default to
+	 * status 200 (OK, ignoring the ranges). Switch to status 206 (Partial
+	 * Content) if there is at least one partially valid range. Switch to
+	 * status 416 (Range Not Satisfiable) if there are no partially valid
+	 * ranges at all. */
 	array = g_array_new (FALSE, FALSE, sizeof (SoupRange));
 	for (r = range_list; r; r = r->next) {
 		SoupRange cur;
@@ -1205,30 +1232,44 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
 			cur.start = g_ascii_strtoull (spec, &end, 10);
 			if (*end == '-')
 				end++;
-			if (*end) {
+			if (*end)
 				cur.end = g_ascii_strtoull (end, &end, 10);
-				if (cur.end < cur.start) {
-					status = SOUP_STATUS_OK;
-					break;
-				}
-			} else
+			else
 				cur.end = total_length - 1;
 		}
+
 		if (*end) {
-			status = SOUP_STATUS_OK;
-			break;
-		} else if (check_satisfiable && cur.start >= total_length) {
-			if (status == SOUP_STATUS_OK)
-				status = SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE;
+			/* Junk after the range */
+			is_all_valid = FALSE;
 			continue;
 		}
 
+		if (cur.end < cur.start) {
+			is_all_valid = FALSE;
+			continue;
+		}
+
+		g_assert (cur.start >= 0);
+		if (cur.end >= total_length)
+			cur.end = total_length - 1;
+
+		if (cur.start >= total_length) {
+			/* Range is valid, but unsatisfiable */
+			continue;
+		}
+
+		/* We have at least one (at least partially) satisfiable range */
 		g_array_append_val (array, cur);
 		status = SOUP_STATUS_PARTIAL_CONTENT;
 	}
 	soup_header_free_list (range_list);
 
 	if (status != SOUP_STATUS_PARTIAL_CONTENT) {
+		g_assert (status == SOUP_STATUS_OK);
+
+		if (is_all_valid && check_satisfiable)
+			status = SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE;
+
 		g_array_free (array, TRUE);
 		return status;
 	}
@@ -1244,6 +1285,7 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
 			if (cur->start <= prev->end) {
 				prev->end = MAX (prev->end, cur->end);
 				g_array_remove_index (array, i);
+				i--;
 			}
 		}
 	}
@@ -1278,7 +1320,7 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
  * Beware that even if given a @total_length, this function does not
  * check that the ranges are satisfiable.
  *
- * #SoupServer has built-in handling for range requests. If your
+ * [class@Server] has built-in handling for range requests. If your
  * server handler returns a %SOUP_STATUS_OK response containing the
  * complete response body (rather than pausing the message and
  * returning some of the response body later), and there is a Range
@@ -1361,7 +1403,7 @@ soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
 		}
 	}
 
-	soup_message_headers_replace_common (hdrs, SOUP_HEADER_RANGE, header->str);
+	soup_message_headers_replace_common (hdrs, SOUP_HEADER_RANGE, header->str, SOUP_HEADER_VALUE_TRUSTED);
 	g_string_free (header, TRUE);
 }
 
@@ -1482,7 +1524,7 @@ soup_message_headers_set_content_range (SoupMessageHeaders  *hdrs,
 		header = g_strdup_printf ("bytes %" G_GINT64_FORMAT "-%"
 					  G_GINT64_FORMAT "/*", start, end);
 	}
-	soup_message_headers_replace_common (hdrs, SOUP_HEADER_CONTENT_RANGE, header);
+	soup_message_headers_replace_common (hdrs, SOUP_HEADER_CONTENT_RANGE, header, SOUP_HEADER_VALUE_TRUSTED);
 	g_free (header);
 }
 
@@ -1538,6 +1580,46 @@ parse_content_foo (SoupMessageHeaders *hdrs,
 	return TRUE;
 }
 
+static inline gboolean
+is_valid_leading_or_trailingcharacter_for_filename (guchar c)
+{
+        return !g_ascii_isspace (c) && c != '\r' && c != '\n' && c != '/' && c != '\\' && c != '.';
+}
+
+static char *
+sanitize_filename (const char *value)
+{
+        char *filename;
+        guchar *c;
+        gsize len;
+
+        if (!value || !value[0])
+                return NULL;
+
+        filename = g_strdup (value);
+
+        /* Remove leading invalid characters */
+        for (c = (guchar*)filename; *c && !is_valid_leading_or_trailingcharacter_for_filename (*c); c++)
+                ;
+        memmove (filename, c, strlen ((gchar *)c) + 1);
+
+        /* Remove trailing invalid characters */
+        len = strlen (filename);
+        while (len--) {
+                if (is_valid_leading_or_trailingcharacter_for_filename ((guchar)filename[len]))
+                        break;
+                filename[len] = '\0';
+        }
+
+        /* Replace all other invalid characters with '_' */
+        for (c = (guchar*)filename; *c; c++) {
+                if (strchr ("\"~*/:<>?\\|\r\n", *c) || g_ascii_isspace (*c) || g_ascii_iscntrl (*c))
+                        *c = '_';
+        }
+
+        return filename;
+}
+
 static void
 set_content_foo (SoupMessageHeaders *hdrs,
                  SoupHeaderName      header_name,
@@ -1552,12 +1634,17 @@ set_content_foo (SoupMessageHeaders *hdrs,
 	if (params) {
 		g_hash_table_iter_init (&iter, params);
 		while (g_hash_table_iter_next (&iter, &key, &value)) {
+                        char *filename = NULL;
+
 			g_string_append (str, "; ");
-			soup_header_g_string_append_param (str, key, value);
+                        if (g_strcmp0 (key, "filename") == 0)
+                                filename = sanitize_filename (value);
+			soup_header_g_string_append_param (str, key, filename ? filename : value);
+                        g_free (filename);
 		}
 	}
 
-	soup_message_headers_replace_common (hdrs, header_name, str->str);
+	soup_message_headers_replace_common (hdrs, header_name, str->str, SOUP_HEADER_VALUE_UNTRUSTED);
 	g_string_free (str, TRUE);
 }
 
@@ -1655,15 +1742,18 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
 				disposition, params))
 		return FALSE;
 
-	/* If there is a filename parameter, make sure it contains
-	 * only a single path component
-	 */
+	/* If there is a filename parameter, make sure it contains only valid characters */
 	if (params && g_hash_table_lookup_extended (*params, "filename",
 						    &orig_key, &orig_value)) {
-		char *filename = strrchr (orig_value, '/');
+                if (orig_value) {
+                        char *filename = sanitize_filename (orig_value);
 
-		if (filename)
-			g_hash_table_insert (*params, g_strdup (orig_key), filename + 1);
+                        if (filename)
+                                g_hash_table_insert (*params, g_strdup (orig_key), g_steal_pointer (&filename));
+                } else {
+                        /* filename with no value isn't valid. */
+                        g_hash_table_remove (*params, "filename");
+                }
 	}
 	return TRUE;
 }
