@@ -31,7 +31,7 @@
  * [method@MultipartInputStream.next_part] before reading. Responses
  * which are not wrapped will be treated like non-multipart responses.
  *
- * Note that although #SoupMultipartInputStream is a [class@Gio.InputStream],
+ * Note that although [class@MultipartInputStream] is a [class@Gio.InputStream],
  * you should not read directly from it, and the results are undefined
  * if you do.
  **/
@@ -376,7 +376,7 @@ soup_multipart_input_stream_read_headers (SoupMultipartInputStream  *multipart,
 							    /* blocking */ TRUE, &got_lf, cancellable, error);
 
 		if (nread <= 0)
-			break;
+			return FALSE;
 
 		g_byte_array_append (priv->meta_buf, read_buf, nread);
 
@@ -431,7 +431,7 @@ soup_multipart_input_stream_read_headers (SoupMultipartInputStream  *multipart,
  * @msg: the #SoupMessage the response is related to.
  * @base_stream: the #GInputStream returned by sending the request.
  *
- * Creates a new #SoupMultipartInputStream that wraps the
+ * Creates a new [class@MultipartInputStream] that wraps the
  * [class@Gio.InputStream] obtained by sending the [class@Message].
  *
  * Reads should not be done directly through this object, use the input streams
@@ -459,7 +459,7 @@ soup_multipart_input_stream_new (SoupMessage  *msg,
  * Obtains an input stream for the next part.
  *
  * When dealing with a multipart response the input stream needs to be wrapped
- * in a #SoupMultipartInputStream and this function or its async counterpart
+ * in a [class@MultipartInputStream] and this function or its async counterpart
  * need to be called to obtain the first part for reading.
  *
  * After calling this function,
@@ -468,6 +468,9 @@ soup_multipart_input_stream_new (SoupMessage  *msg,
  * the part; a new call to this function should be done at that point,
  * to obtain the next part.
  *
+ * @error will only be set if an error happens during a read, %NULL
+ * is a valid return value otherwise.
+ *
  * Returns: (nullable) (transfer full): a new #GInputStream, or
  *   %NULL if there are no more parts
  */
@@ -581,7 +584,7 @@ soup_multipart_input_stream_next_part_finish (SoupMultipartInputStream	*multipar
  * Obtains the headers for the part currently being processed.
  *
  * Note that the [struct@MessageHeaders] that are returned are owned by the
- * #SoupMultipartInputStream and will be replaced when a call is made to
+ * [class@MultipartInputStream] and will be replaced when a call is made to
  * [method@MultipartInputStream.next_part] or its async counterpart, so if
  * keeping the headers is required, a copy must be made.
  *
