@@ -83,7 +83,6 @@ test_init (int argc, char **argv, GOptionEntry *entries)
 	g_set_prgname (name);
 
 	g_test_init (&argc, &argv, NULL);
-	g_test_set_nonfatal_assertions ();
 	g_test_bug_base ("https://bugzilla.gnome.org/");
 
 	opts = g_option_context_new (NULL);
@@ -143,6 +142,8 @@ debug_printf (int level, const char *format, ...)
 	va_start (args, format);
 	g_vprintf (format, args);
 	va_end (args);
+
+	fflush (stdout);
 }
 
 gboolean
@@ -605,9 +606,11 @@ static gboolean
 add_listener_in_thread (gpointer user_data)
 {
 	AddListenerData *data = user_data;
+	GUri *uri;
 
-	data->uri = add_listener (data->server, data->scheme, data->host);
+	uri = add_listener (data->server, data->scheme, data->host);
 	g_mutex_lock (&data->mutex);
+	data->uri = uri;
 	g_cond_signal (&data->cond);
 	g_mutex_unlock (&data->mutex);
 
@@ -639,9 +642,9 @@ soup_test_server_get_uri (SoupServer    *server,
 		data.host = host;
 		data.uri = NULL;
 
-		g_mutex_lock (&data.mutex);
 		soup_add_completion (context, add_listener_in_thread, &data);
 
+		g_mutex_lock (&data.mutex);
 		while (!data.uri)
 			g_cond_wait (&data.cond, &data.mutex);
 
@@ -727,12 +730,8 @@ async_as_sync_callback (GObject      *object,
 			gpointer      user_data)
 {
 	AsyncAsSyncData *data = user_data;
-	GMainContext *context;
 
 	data->result = g_object_ref (result);
-	context = g_main_loop_get_context (data->loop);
-	while (g_main_context_pending (context))
-		g_main_context_iteration (context, FALSE);
 	g_main_loop_quit (data->loop);
 }
 
@@ -772,16 +771,10 @@ soup_test_request_send (SoupSession   *session,
 	stream = soup_session_send_finish (session, data.result, error);
 
 	if (flags & SOUP_TEST_REQUEST_CANCEL_AFTER_SEND_FINISH) {
-		GMainContext *context;
-
                 if (flags & SOUP_TEST_REQUEST_CANCEL_BY_SESSION)
                         soup_session_cancel_message (session, msg);
                 else
                         g_cancellable_cancel (cancellable);
-
-		context = g_main_loop_get_context (data.loop);
-		while (g_main_context_pending (context))
-			g_main_context_iteration (context, FALSE);
 	}
 
 	g_main_loop_unref (data.loop);
